clear all
close all
model = 'full'; % 'full' or 'reduced'
testfun = 'cubic'; % 'cubic' or 'Franke'
gradient = 'exact'; % 'exact' or 'approximated'
N = 20;
%
x = linspace(0,1,N);
y = linspace(0,1,N);
[x,y] = ndgrid(x,y);
x(2:N-1,2:N-1) = x(2:N-1,2:N-1)+2*(rand(N-2)-0.5)/(2*N);
y(2:N-1,2:N-1) = y(2:N-1,2:N-1)+2*(rand(N-2)-0.5)/(2*N);
x = x(:);
y = y(:);
if (strcmp(testfun,'cubic'))
  f_fun = @(x,y) 1+x+y+x.^2+x.*y+y.^2+x.^3+x.^2.*y+x.*y.^2+y.^3;
  fx_fun = @(x,y) 1+2*x+y+3*x.^2+2*x.*y+y.^2;
  fy_fun = @(x,y) 1+x+2*y+x.^2+2*y.*x+3*y.^2;
else % strcmp(testfun,'Franke')
  f_fun = @(x,y) franke(x,y);
  fx_fun = @(x,y) 3/4*(-2*(9*x-2)/4)*9.*exp(-((9*x-2).^2+(9*y-2).^2)/4) +...
      3/4*(-2*(9*x+1)/49)*9.*exp(-((9*x+1).^2)/49-(9*y+1)/10) + ...
      1/2*(-2*(9*x-7)/4)*9.*exp(-((9*x-7).^2+(9*y-3).^2)/4) -...
      1/5*(-2*(9*x-4))*9.*exp(-(9*x-4).^2-(9*y-7).^2);
  fy_fun = @(x,y) 3/4*(-2*(9*y-2)/4)*9.*exp(-((9*x-2).^2+(9*y-2).^2)/4) +...
      3/4*(-9/10)*exp(-((9*x+1).^2)/49-(9*y+1)/10) + ...
      1/2*(-2*(9*y-3)/4)*9.*exp(-((9*x-7).^2+(9*y-3).^2)/4) -...
      1/5*(-2*(9*y-7))*9.*exp(-(9*x-4).^2-(9*y-7).^2);
end
dt = DelaunayTri(x,y);
triplot(dt)
tri = dt.Triangulation;
ell = size(tri,1);
% compute in any case approximated gradients
f_vec = f_fun(x,y);
fx_vec = zeros(size(x));
fy_vec = zeros(size(y));
nodecount = zeros(size(x));
for l = 1:ell
  localx = x(tri(l,:));
  localy = y(tri(l,:));
  localf = f_vec(tri(l,:));
% plane c(1)+c(2)*x+c(3)*y-z=0 through triangle l
  c = [ones(3,1),localx(:),localy(:)]\localf(:);
  nodecount(tri(l,:)) = nodecount(tri(l,:))+1;
  fx_vec(tri(l,:)) = fx_vec(tri(l,:))+c(2);
  fy_vec(tri(l,:)) = fy_vec(tri(l,:))+c(3);
end
fx_vec = fx_vec./nodecount;
fy_vec = fy_vec./nodecount;
HCT.x = [];
HCT.y = [];
HCT.coefs = [];
for l = 1:ell
  localx = x(tri(l,:));
  localy = y(tri(l,:));
% analytic gradient
  hct = triangle(localx,localy,f_fun,fx_fun,fy_fun,model);
  localf = f_vec(tri(l,:));
  localfx = fx_vec(tri(l,:));
  localfy = fy_vec(tri(l,:));
% approximated gradient
  if (strcmp(gradient,'approximated'))
% use approximated gradients
    hct = triangle(localx,localy,localf,localfx,localfy,model);
  end
  HCT.x = [HCT.x;hct.x];
  HCT.y = [HCT.y;hct.y];
  HCT.coefs = [HCT.coefs;hct.coefs];
end
xx = linspace(0,1,50);
yy = linspace(0,1,50);
[XX,YY] = ndgrid(xx,yy);
FF = zeros(size(XX));
pl = pointLocation(dt,XX(:),YY(:));
for i = 1:length(XX(:))
  localx = x(tri(pl(i),:));
  localy = y(tri(pl(i),:));
  j = whichsubtriangle(localx,localy,XX(i),YY(i));
  a = HCT.coefs(3*(pl(i)-1)+j,:);
  FF(i) = a*[1;XX(i);YY(i);XX(i)^2;XX(i)*YY(i);YY(i)^2;XX(i)^3;XX(i)^2*YY(i);XX(i)*YY(i)^2;YY(i)^3];
end
F = f_fun(XX,YY);
mesh(XX,YY,F)
title('Original function')
figure
mesh(XX,YY,FF)
title(sprintf('%s HCT interpolation',model))
hold on
plot3(x,y,f_vec,'*')
errorHCT = norm(FF(:)-F(:),inf)
v = f_fun(dt.X(:,1),dt.X(:,2));
F1 = TriScatteredInterp(dt,v,'natural');
FFF = F1(XX,YY);
figure
mesh(XX,YY,FFF)
title('TriScatteredInterp interpolation')
errorTriScatteredInterp = norm(FFF(:)-F(:),inf)
